
using System.Collections.Generic;

using Jypeli;

namespace OilSaber;

/// @author konst
/// @version 16.02.2025
/// <summary>
/// 
/// </summary>
public class OilSaber : PhysicsGame
{
    /// <summary>
    /// Määritellään lista pelaajan painikkeille
    /// </summary>
    private HashSet<Key> _painikkeet = new HashSet<Key>();
    
    
    /// <summary>
    /// laskuri pelaajan Hp:lle
    /// </summary>
    private int _pelaajaHp = 3;
    
    
    /// <summary>
    /// Lista möröille
    /// </summary>
    private List<PhysicsObject> _morkot = new List<PhysicsObject>();
    
    
    /// <summary>
    /// Lista Morko2:llie
    /// </summary>
    private List<PhysicsObject> _morkot2 = new List<PhysicsObject>();
    
    
    /// <summary>
    /// tehdään vihollisista globaaleja muuttujia, jotta niitä voidaan käyttää aliohjelmissa
    /// </summary>
    private PhysicsObject _morko2;
    
    
    
    /// <summary>
    /// Tehdään pelaajasta globaali muuttuja, jotta sitä voidaan käyttää aliohjelmissa
    /// </summary>
    private PhysicsObject _pelaaja;


    /// <summary>
    /// Tehdään pisteistä globaali muuttuja, jotta sitä voidaan käyttää aliohjelmissa
    /// </summary>
    private int _pisteet;
    private Label _pistelaskuri;
    private Label _pelaajaHpLaskuri;
    
    public override void Begin()
    {
        Level.Background.Color = Color.Black;
        LisaaPelaaja();
        Pistelaskuri();
        _Tarkistamorot();
        MorkoSuunnistus();
        UpdateLiike();
        _kuuntelepainikkeet();
        
        //Pelaajan hp laskuri
        _pelaajaHpLaskuri = new Label();
        _pelaajaHpLaskuri.TextColor = Color.White;
        _pelaajaHpLaskuri.BorderColor = Color.Black;
        _pelaajaHpLaskuri.Position = new Vector(Screen.Right -100, Screen.Top - 50);
        _pelaajaHpLaskuri.Text = "Hp:" + _pelaajaHp;
        Add(_pelaajaHpLaskuri);
        
        //ajastin tarkistaa joka 1,5s välein onko mörköjä jäljellä ja kutsuu tarvittaessa "tarkistamorot"
        Timer onkoVihollista = new Timer();
        onkoVihollista.Interval = 1.5;
        onkoVihollista.Timeout += _Tarkistamorot;
        onkoVihollista.Start();
    }
    
    
    /// <summary>
    /// Pelin painikkeiden määrrittely
    /// </summary>
    private void _kuuntelepainikkeet()
    {
        Keyboard.Listen(Key.Up, ButtonState.Pressed, Lisaasuunta, "liikuta pelaaja ylös", Key.Up);
        Keyboard.Listen(Key.Up, ButtonState.Released, Poistasuunta, "pysäytä pelaaja", Key.Up);

        Keyboard.Listen(Key.Down, ButtonState.Pressed, Lisaasuunta, "liikuta pelaaja alas", Key.Down);
        Keyboard.Listen(Key.Down, ButtonState.Released, Poistasuunta, "pysäytä pelaaja", Key.Down);

        Keyboard.Listen(Key.Left, ButtonState.Pressed, Lisaasuunta, "liikuta pelaaja vasemmalle", Key.Left);
        Keyboard.Listen(Key.Left, ButtonState.Released, Poistasuunta, "pyäytä pelaaja", Key.Left);

        Keyboard.Listen(Key.Right, ButtonState.Pressed, Lisaasuunta, "liikuta pelaaja oikealle", Key.Right);
        Keyboard.Listen(Key.Right, ButtonState.Released, Poistasuunta, "pyäytä pelaaja", Key.Right);

        //ampumisen painikkeen määrrittely
        Keyboard.Listen(Key.Space, ButtonState.Pressed, PelaajaAmmu, "pelaaja ampuu");

        // pelistä poistuminen
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }
    
    
    /// <summary>
    /// ajastin mörköjen suunnan vaihteluun
    /// </summary>
    private void MorkoSuunnistus()
    {
        Timer morkoSuunta = new Timer();
        morkoSuunta.Interval = 3;
        morkoSuunta.Timeout += VaihdaMorkojenSuunta;
        morkoSuunta.Start();
    }
    
    
    /// <summary>
    /// Pistelaskuri
    /// </summary>
    private void Pistelaskuri()
    {
        _pistelaskuri = new Label();
        _pistelaskuri.TextColor = Color.LightYellow;
        _pistelaskuri.BorderColor = Color.Black;
        _pistelaskuri.Position = new Vector(Screen.Left + 100, Screen.Top - 50);
        _pistelaskuri.Text = "Pisteet:0";
        Add(_pistelaskuri);
    }
    
    
    /// <summary>
    /// pelaaja
    /// </summary>
    private void LisaaPelaaja()
    {
        _pelaaja = new PhysicsObject(45, 45, Shape.Rectangle);
        _pelaaja.Color = Color.Blue;
        _pelaaja.Tag = "pelaaja";
        Add(_pelaaja);
        _pelaaja.MaxVelocity = 400;
    }
    
    
    /// <summary>
    /// pelaajan hp joka tyhjentyessään tuhoaa pelaajan(Morko2)
    /// </summary>
    /// <param name="morko2"></param>
    /// <param name="toinen"></param>
    private void _TarkistaHp2(PhysicsObject morko2, PhysicsObject toinen)
    {
        if (toinen.Tag.ToString() == "pelaaja")
        {
            _pelaajaHp--;
            if (_pelaajaHp <= 0)
            {
                Remove(_pelaaja);
                MessageDisplay.Add("Tuhouduit!");
            }
            else
            {
                MessageDisplay.Add(_pelaajaHp + "/3 HP");
            }
        }
    }
    
    
    /// <summary>
    /// pelaajan hp joka tyhjentyessään tuhoaa pelaajan
    /// </summary>
    /// <param name="morko"></param>
    /// <param name="toinen"></param>
    private void _TarkistaHp(PhysicsObject morko, PhysicsObject toinen)
    {
        if (toinen.Tag.ToString()== "pelaaja")
        {
            _pelaajaHp--;
            if (_pelaajaHp <= 0)
            {
                Remove(_pelaaja);
                MessageDisplay.Add("Tuhouduit!");
                ConfirmExit();
            }
            else
            {
                MessageDisplay.Add(_pelaajaHp + "/3 HP");
            }
            _pelaajaHpLaskuri.Text = "HP:" + _pelaajaHp;
        }
    }
    
    
    /// <summary>
    /// funktio jolla mörkö2 tuhotaan
    /// </summary>
    /// <param name="morko2"></param>
    /// <param name="toinen"></param>
    private void Morkotuhoa2(PhysicsObject morko2, PhysicsObject toinen)
    {
        if (toinen.Tag.ToString() == "luoti")
        {
            Remove(morko2);
            _morkot2.Remove(morko2);
            Remove(toinen);
        }
        MessageDisplay.Clear();
        MessageDisplay.Add("+10p");
        _pisteet += 15;
        _pistelaskuri.Text = "Pisteet:" + _pisteet; 
    }
    
    
    /// <summary>
    /// funktio jolla möröt tuhotaan
    /// </summary>
    /// <param name="morko"></param>
    /// <param name="toinen"></param>
    private void Morkotuhoa(PhysicsObject morko, PhysicsObject toinen)
    {
        if (toinen.Tag.ToString()== "luoti")
        {
                
            Remove(morko);
            _morkot.Remove(morko);
            Remove(toinen);

        }
        MessageDisplay.Clear();
        MessageDisplay.Add("+10p");
        _pisteet += 10;
        _pistelaskuri.Text = "Pisteet:" + _pisteet; 
    }
    
    
    /// <summary>
    /// pelaajan hp joka tyhjentyessään tuhoaa pelaajan(Morko2ampuu)
    /// </summary>
    /// <param name="uusiLuoti2"></param>
    /// <param name="toinen"></param>
    private void _TarkistaHP3(PhysicsObject uusiLuoti2, PhysicsObject toinen)
    {
        if (toinen.Tag.ToString() == "pelaaja")
        {
            _pelaajaHp--;
            if (_pelaajaHp <= 0)
            {
                Remove(_pelaaja);
                MessageDisplay.Add("Tuhouduit!");
                ConfirmExit();
            }
            else
            {
                MessageDisplay.Add(_pelaajaHp + "/3 HP");
            }
            _pelaajaHpLaskuri.Text = "HP:" + _pelaajaHp;
        }
    }
    
    
    /// <summary>
    /// määritellään Mörkö2 ampuminen pelaajaa kohti
    /// </summary>
    /// <param name="morko2"></param>
    private void Ammu2(PhysicsObject morko2)
    {
        PhysicsObject uusiLuoti2 = new PhysicsObject(10, 10, Shape.Diamond);
        uusiLuoti2.Color = Color.Pink;
        Vector suunta = (_pelaaja.Position - morko2.Position).Normalize(); //laskee pelaajan vektorin
        uusiLuoti2.Position = morko2.Position + suunta;
        uusiLuoti2.Velocity = suunta * 700;
        uusiLuoti2.CollisionIgnoreGroup = 2; // Estetään törmäykset Mörköjen kanssa kanssa
        uusiLuoti2.Tag = "luoti2"; //tunniste törmäystä varten
        AddCollisionHandler(uusiLuoti2, _TarkistaHP3);// mahdollistetaan pelaajan vahingoittaminen
        Add(uusiLuoti2);
        Timer.SingleShot(3.0, () => Remove(uusiLuoti2)); // Poistetaan luoti kun se menee liian pitkälle
    }
    
    
    /// <summary>
    /// Kutsutaan Ammu2 jokaiselle morko2:lle
    /// </summary>
    private void _Morko2ampuu()
    {
        foreach(var morko2 in _morkot2)
        {
            Ammu2(morko2);  
        }
            
    }
    
    
    /// <summary>
    ///  määritellään luoti  ja sen liikkuminen
    /// </summary>
    private void Ammu()
    {
        PhysicsObject uusiLuoti = new PhysicsObject(5, 10, Shape.Diamond);
        uusiLuoti.Color = Color.Red;
        uusiLuoti.Position = _pelaaja.Position + new Vector(0, 20);
        uusiLuoti.Velocity = new Vector(0, 700);
        uusiLuoti.CollisionIgnoreGroup = 1; // Estetään törmäykset pelaajan kanssa
        uusiLuoti.Tag = "luoti"; //tunniste törmäystä varten
        Add(uusiLuoti);
        Timer.SingleShot(3.0, () => Remove(uusiLuoti)); // Poistetaan luoti kun se menee liian korkealle
    }
    
    
    /// <summary>
    /// pelaaja ampuu
    /// </summary>
    private void PelaajaAmmu()
    { 
        Ammu();
    }
    
    
    /// <summary>
    /// Päivittää painettujen painikkeiden listan ja määrittelee miten pelaaja liikkuu
    /// </summary>
    private void UpdateLiike()
    {
        Vector liike = new Vector();

        if (_painikkeet.Contains(Key.Up)) liike += new Vector(0, 300);
        if (_painikkeet.Contains(Key.Down)) liike += new Vector(0, -300);
        if (_painikkeet.Contains(Key.Left)) liike += new Vector(-300, 0);
        if (_painikkeet.Contains(Key.Right)) liike += new Vector(300, 0);

        _pelaaja.Velocity = liike;
    }
    
    
    /// <summary>
    /// lista johon lisätään painetut painikkeet ja päivitetään tilanne
    /// </summary>
    /// <param name="painike"></param>
    private void Lisaasuunta(Key painike)
    {
        _painikkeet.Add(painike);
        UpdateLiike();
    }

        
/// <summary>
/// lista josta poistetaan vapautetu painikkeet ja päivitetään tilanne
/// </summary>
/// <param name="painike"></param>
    private void Poistasuunta(Key painike)
    {
        _painikkeet.Remove(painike);
        UpdateLiike();
    }
    
    
    /// <summary>
    /// funktio tarkistaa onko mörköjä jäljellä. Jos ei ole, niin se lisää niitä
    /// </summary>
    private void _Tarkistamorot()
    {
        if (_morkot.Count == 00)
        {
            LisaaMorko(5);
            
            if (_morkot2.Count == 00)
            {
            }

            {
                LisaaMorko2(3);
            }
        }
    }
    
    
    /// <summary>
    /// funktio mörköjen suunnan vaihtamiselle
    /// </summary>
    private void VaihdaMorkojenSuunta()
    {
        foreach (var morko in _morkot)
        {
            if (morko.Left < Level.Left ||
                morko.Right > Level.Right) //jos möröt ovat ruudun reunalla, niin ne kääntyvät
            {
                morko.Velocity = -morko.Velocity;
            }

            if (morko.Top > Level.Top ||
                morko.Bottom < Level.Bottom) //jos möröt ovat ruudun reunalla, niin ne kääntyvät
            {
                morko.Velocity = -morko.Velocity;
            }

            if (_pelaaja != null)
            {
                Vector suunta = (_pelaaja.Position - morko.Position).Normalize(); //laskee pelaajan vektorin
                morko.Move(suunta * 200); //lähettää möröt pelaajaa kohti
            }
        }
        foreach (var morko2 in _morkot2)
        {
            if (morko2.Left < Level.Left ||
                morko2.Right > Level.Right) //jos möröt ovat ruudun reunalla, niin ne kääntyvät
            {
                morko2.Velocity = -morko2.Velocity;
            }

            if (morko2.Top > Level.Top ||
                morko2.Bottom < Level.Bottom) //jos möröt ovat ruudun reunalla, niin ne kääntyvät
            {
                morko2.Velocity = -morko2.Velocity;
            }

            if (_pelaaja != null)
            {
                Vector suunta = (_pelaaja.Position - morko2.Position).Normalize(); //laskee pelaajan vektorin
                morko2.Move(suunta * 200); //lähettää möröt pelaajaa kohti
            }
        }

    }
    
    /// <summary>
    /// Morko2
    /// </summary>
    /// <param name="maara"></param>
    private void LisaaMorko2(int maara)
    {
        for (int i = 0; i < maara; i++)
        {
            _morko2 = new PhysicsObject(70, 70, Shape.Circle);
            _morko2.Color = Color.OrangeRed;
            _morko2.Tag = "Morko2";
            double x = RandomGen.NextDouble(Level.Left, Level.Right);
            double y = Level.Top;
            _morko2.CollisionIgnoreGroup = 2;
            _morko2.Position = new Vector(x, y); // Aseta satunnainen sijainti
            _morko2.Velocity = new Vector(0, -200);
            AddCollisionHandler(_morko2, Morkotuhoa2); //törmäyskäsittelijä jolla pelaaja voi tuhota mörköjä
            AddCollisionHandler(_morko2, _TarkistaHp2); //törmäyskäsittelijä, jolla pelaaja voidaan tuhota
            Add(_morko2);
            _morkot2.Add(_morko2);
                
            //ajastin ampumiselle
            Timer morko2AmpumisAika = new Timer();
            morko2AmpumisAika.Interval = 3;
            if (_pelaaja != null || _morko2 != null)
            {
                morko2AmpumisAika.Timeout += _Morko2ampuu;
            }
            morko2AmpumisAika.Start();
        }
    }
    /// <summary>
    /// Mörkö
    /// </summary>
    /// <param name="maara"></param>
    private void LisaaMorko(int maara)
    {
        for (int i = 0; i < maara; i++)
        {
            PhysicsObject morko = new PhysicsObject(30, 30, Shape.Circle);
            morko.Color = Color.Green;
            morko.Tag = "Morko";
            double x = RandomGen.NextDouble(Level.Left, Level.Right);
            double y = Level.Top;
            morko.CollisionIgnoreGroup = 2;
            morko.Position = new Vector(x, y); // Aseta satunnainen sijainti
            morko.Velocity = new Vector(0, -100);
            AddCollisionHandler(morko, Morkotuhoa); //törmäyskäsittelijä jolla pelaaja voi tuhota mörköjä
            AddCollisionHandler(morko, _TarkistaHp); //törmäyskäsittelijä, jolla pelaaja voidaan tuhota
            Add(morko);
            _morkot.Add(morko);
        }
    }
}
        

        

    
